@@ -1,3 +1,7 @@
+# Licensed to Elasticsearch B.V under one or more agreements.
+# Elasticsearch B.V licenses this file to you under the Apache 2.0 License.
+# See the LICENSE file in the project root for more information
+
 require "cgi"
 require "multi_json"
 
@@ -28,7 +32,8 @@ module Elasticsearch
       :format,                        # Search, Cat, ...
       :pretty,                        # Pretty-print the response
       :human,                         # Return numeric values in human readable format
-      :filter_path                    # Filter the JSON response
+      :filter_path,                   # Filter the JSON response
+      :opaque_id                      # Use X-Opaque-Id
     ]
 
     HTTP_GET          = 'GET'.freeze
