@@ -1,3 +1,7 @@
+# Licensed to Elasticsearch B.V under one or more agreements.
+# Elasticsearch B.V licenses this file to you under the Apache 2.0 License.
+# See the LICENSE file in the project root for more information
+
 module Elasticsearch
   module Transport
     module Transport
@@ -15,8 +19,8 @@ module Elasticsearch
           # @return [Response]
           # @see    Transport::Base#perform_request
           #
-          def perform_request(method, path, params={}, body=nil, headers=nil)
-            super do |connection,url|
+          def perform_request(method, path, params={}, body=nil, headers=nil, opts={})
+            super do |connection, url|
               connection.connection.url = url
 
               case method
